@@ -1,7 +1,7 @@
-#include <Elementary.h>
 #ifdef HAVE_CONFIG_H
 # include "elementary_config.h"
 #endif
+#include <Elementary.h>
 #ifndef ELM_LIB_QUICKLAUNCH
 
 void
@@ -63,19 +63,13 @@ my_fl_8(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 void
 test_flip(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
-   Evas_Object *win, *bg, *bx, *bx2, *fl, *o, *bt, *ly;
+   Evas_Object *win, *bx, *bx2, *fl, *o, *bt, *ly;
    char buf[PATH_MAX];
 
-   win = elm_win_add(NULL, "flip", ELM_WIN_BASIC);
-   elm_win_title_set(win, "Flip");
+   win = elm_win_util_standard_add("flip", "Flip");
    elm_win_focus_highlight_enabled_set(win, EINA_TRUE);
    elm_win_autodel_set(win, EINA_TRUE);
 
-   bg = elm_bg_add(win);
-   elm_win_resize_object_add(win, bg);
-   evas_object_size_hint_weight_set(bg, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
-   evas_object_show(bg);
-
    bx = elm_box_add(win);
    evas_object_size_hint_weight_set(bx, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    elm_win_resize_object_add(win, bx);
@@ -91,7 +85,7 @@ test_flip(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info _
    evas_object_size_hint_weight_set(o, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    snprintf(buf, sizeof(buf), "%s/images/%s", elm_app_data_dir_get(), "sky_01.jpg");
    elm_bg_file_set(o, buf, NULL);
-   elm_flip_content_front_set(fl, o);
+   elm_object_part_content_set(fl, "front", o);
    evas_object_show(o);
 
    ly = elm_layout_add(win);
@@ -99,7 +93,7 @@ test_flip(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info _
    elm_layout_file_set(ly, buf, "layout");
    evas_object_size_hint_align_set(ly, EVAS_HINT_FILL, EVAS_HINT_FILL);
    evas_object_size_hint_weight_set(ly, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
-   elm_flip_content_back_set(fl, ly);
+   elm_object_part_content_set(fl, "back", ly);
    evas_object_show(ly);
 
    bt = elm_button_add(ly);
@@ -264,33 +258,33 @@ test_flip2(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info
    elm_box_pack_end(o, li);
    evas_object_show(li);
 
-   elm_flip_content_front_set(fl, o);
+   elm_object_part_content_set(fl, "front", o);
    evas_object_show(o);
 
    en = elm_entry_add(win);
    elm_entry_scrollable_set(en, EINA_TRUE);
    elm_entry_line_wrap_set(en, ELM_WRAP_NONE);
    snprintf(buf, sizeof(buf),
-            "This is an entry widget in this window that<br>"
-            "uses markup <b>like this</> for styling and<br>"
-            "formatting <em>like this</>, as well as<br>"
-            "<a href=X><link>links in the text</></a>, so enter text<br>"
-            "in here to edit it. By the way, links are<br>"
-            "called <a href=anc-02>Anchors</a> so you will need<br>"
-            "to refer to them this way.<br>"
-            "<br>"
+            "This is an entry widget in this window that<br/>"
+            "uses markup <b>like this</> for styling and<br/>"
+            "formatting <em>like this</>, as well as<br/>"
+            "<a href=X><link>links in the text</></a>, so enter text<br/>"
+            "in here to edit it. By the way, links are<br/>"
+            "called <a href=anc-02>Anchors</a> so you will need<br/>"
+            "to refer to them this way.<br/>"
+            "<br/>"
 
             "Also you can stick in items with (relsize + ascent): "
             "<item relsize=16x16 vsize=ascent href=emoticon/evil-laugh></item>"
             " (full) "
             "<item relsize=16x16 vsize=full href=emoticon/guilty-smile></item>"
-            " (to the left)<br>"
+            " (to the left)<br/>"
 
             "Also (size + ascent): "
             "<item size=16x16 vsize=ascent href=emoticon/haha></item>"
             " (full) "
             "<item size=16x16 vsize=full href=emoticon/happy-panting></item>"
-            " (before this)<br>"
+            " (before this)<br/>"
 
             "And as well (absize + ascent): "
             "<item absize=64x64 vsize=ascent href=emoticon/knowing-grin></item>"
@@ -301,11 +295,11 @@ test_flip2(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info
             " ... end."
             , elm_app_data_dir_get()
            );
-   elm_entry_entry_set(en, buf);
+   elm_object_text_set(en, buf);
    evas_object_size_hint_align_set(en, EVAS_HINT_FILL, EVAS_HINT_FILL);
    evas_object_size_hint_weight_set(en, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
 
-   elm_flip_content_back_set(fl, en);
+   elm_object_part_content_set(fl, "back", en);
    evas_object_show(en);
 
    evas_object_show(fl);
@@ -373,18 +367,12 @@ test_flip2(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info
 void
 test_flip3(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
-   Evas_Object *win, *bg, *bx, *bx2, *fl, *fl_f, *fl_b, *o, *bt, *fr;
+   Evas_Object *win, *bx, *bx2, *fl, *fl_f, *fl_b, *o, *bt, *fr;
    char buf[PATH_MAX];
 
-   win = elm_win_add(NULL, "flip3", ELM_WIN_BASIC);
-   elm_win_title_set(win, "Flip Flip");
+   win = elm_win_util_standard_add("flip3", "Flip Flip");
    elm_win_autodel_set(win, EINA_TRUE);
 
-   bg = elm_bg_add(win);
-   elm_win_resize_object_add(win, bg);
-   evas_object_size_hint_weight_set(bg, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
-   evas_object_show(bg);
-
    bx = elm_box_add(win);
    evas_object_size_hint_weight_set(bx, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    elm_win_resize_object_add(win, bx);
@@ -400,7 +388,7 @@ test_flip3(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info
    elm_object_text_set(fr, "Front");
    evas_object_size_hint_align_set(fr, EVAS_HINT_FILL, EVAS_HINT_FILL);
    evas_object_size_hint_weight_set(fr, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
-   elm_flip_content_front_set(fl, fr);
+   elm_object_part_content_set(fl, "front", fr);
    evas_object_show(fr);
 
    fl_f = elm_flip_add(win);
@@ -414,7 +402,7 @@ test_flip3(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info
    evas_object_size_hint_weight_set(o, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    snprintf(buf, sizeof(buf), "%s/images/%s", elm_app_data_dir_get(), "sky_01.jpg");
    elm_bg_file_set(o, buf, NULL);
-   elm_flip_content_front_set(fl_f, o);
+   elm_object_part_content_set(fl_f, "front", o);
    evas_object_show(o);
 
    o = elm_bg_add(win);
@@ -422,14 +410,14 @@ test_flip3(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info
    evas_object_size_hint_weight_set(o, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    snprintf(buf, sizeof(buf), "%s/images/%s", elm_app_data_dir_get(), "sky_02.jpg");
    elm_bg_file_set(o, buf, NULL);
-   elm_flip_content_back_set(fl_f, o);
+   elm_object_part_content_set(fl_f, "back", o);
    evas_object_show(o);
 
    fr = elm_frame_add(win);
    elm_object_text_set(fr, "Back");
    evas_object_size_hint_align_set(fr, EVAS_HINT_FILL, EVAS_HINT_FILL);
    evas_object_size_hint_weight_set(fr, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
-   elm_flip_content_back_set(fl, fr);
+   elm_object_part_content_set(fl, "back", fr);
    evas_object_show(fr);
 
    fl_b = elm_flip_add(win);
@@ -443,7 +431,7 @@ test_flip3(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info
    evas_object_size_hint_weight_set(o, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    snprintf(buf, sizeof(buf), "%s/images/%s", elm_app_data_dir_get(), "sky_03.jpg");
    elm_bg_file_set(o, buf, NULL);
-   elm_flip_content_front_set(fl_b, o);
+   elm_object_part_content_set(fl_b, "front", o);
    evas_object_show(o);
 
    o = elm_bg_add(win);
@@ -451,7 +439,7 @@ test_flip3(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info
    evas_object_size_hint_weight_set(o, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    snprintf(buf, sizeof(buf), "%s/images/%s", elm_app_data_dir_get(), "sky_04.jpg");
    elm_bg_file_set(o, buf, NULL);
-   elm_flip_content_back_set(fl_b, o);
+   elm_object_part_content_set(fl_b, "back", o);
    evas_object_show(o);
 
    bx2 = elm_box_add(win);
@@ -511,18 +499,12 @@ my_fl_ch(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 void
 test_flip4(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
-   Evas_Object *win, *bg, *bx, *fl, *im, *li, *bt, *rd, *rdg;
+   Evas_Object *win, *bx, *fl, *im, *li, *bt, *rd, *rdg;
    char buf[PATH_MAX];
 
-   win = elm_win_add(NULL, "flip4", ELM_WIN_BASIC);
-   elm_win_title_set(win, "Flip Interactive");
+   win = elm_win_util_standard_add("flip-interactive", "Flip Interactive");
    elm_win_autodel_set(win, EINA_TRUE);
 
-   bg = elm_bg_add(win);
-   elm_win_resize_object_add(win, bg);
-   evas_object_size_hint_weight_set(bg, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
-   evas_object_show(bg);
-
    bx = elm_box_add(win);
    evas_object_size_hint_weight_set(bx, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    elm_win_resize_object_add(win, bx);
@@ -535,14 +517,14 @@ test_flip4(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info
    evas_object_data_set(win, "fl", fl);
 
    elm_flip_interaction_set(fl, ELM_FLIP_INTERACTION_NONE);
-   elm_flip_interacton_direction_enabled_set(fl, ELM_FLIP_DIRECTION_UP, EINA_TRUE);
-   elm_flip_interacton_direction_enabled_set(fl, ELM_FLIP_DIRECTION_DOWN, EINA_TRUE);
-   elm_flip_interacton_direction_enabled_set(fl, ELM_FLIP_DIRECTION_LEFT, EINA_TRUE);
-   elm_flip_interacton_direction_enabled_set(fl, ELM_FLIP_DIRECTION_RIGHT, EINA_TRUE);
-   elm_flip_interacton_direction_hitsize_set(fl, ELM_FLIP_DIRECTION_UP, 0.25);
-   elm_flip_interacton_direction_hitsize_set(fl, ELM_FLIP_DIRECTION_DOWN, 0.25);
-   elm_flip_interacton_direction_hitsize_set(fl, ELM_FLIP_DIRECTION_LEFT, 0.25);
-   elm_flip_interacton_direction_hitsize_set(fl, ELM_FLIP_DIRECTION_RIGHT, 0.25);
+   elm_flip_interaction_direction_enabled_set(fl, ELM_FLIP_DIRECTION_UP, EINA_TRUE);
+   elm_flip_interaction_direction_enabled_set(fl, ELM_FLIP_DIRECTION_DOWN, EINA_TRUE);
+   elm_flip_interaction_direction_enabled_set(fl, ELM_FLIP_DIRECTION_LEFT, EINA_TRUE);
+   elm_flip_interaction_direction_enabled_set(fl, ELM_FLIP_DIRECTION_RIGHT, EINA_TRUE);
+   elm_flip_interaction_direction_hitsize_set(fl, ELM_FLIP_DIRECTION_UP, 0.25);
+   elm_flip_interaction_direction_hitsize_set(fl, ELM_FLIP_DIRECTION_DOWN, 0.25);
+   elm_flip_interaction_direction_hitsize_set(fl, ELM_FLIP_DIRECTION_LEFT, 0.25);
+   elm_flip_interaction_direction_hitsize_set(fl, ELM_FLIP_DIRECTION_RIGHT, 0.25);
    evas_object_show(fl);
 
    im = evas_object_image_filled_add(evas_object_evas_get(win));
@@ -550,7 +532,7 @@ test_flip4(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info
    snprintf(buf, sizeof(buf), "%s/images/%s",
             elm_app_data_dir_get(), "twofish.jpg");
    evas_object_image_file_set(im, buf, NULL);
-   elm_flip_content_front_set(fl, im);
+   elm_object_part_content_set(fl, "front", im);
    evas_object_show(im);
 
 #if 0
@@ -559,7 +541,7 @@ test_flip4(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info
    snprintf(buf, sizeof(buf), "%s/images/%s",
             elm_app_data_dir_get(), "sky_04.jpg");
    evas_object_image_file_set(im, buf, NULL);
-   elm_flip_content_back_set(fl, im);
+   elm_object_part_content_set(fl, "back", im);
    evas_object_show(im);
 #else
    li = elm_list_add(win);
@@ -569,7 +551,7 @@ test_flip4(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info
    elm_list_item_append(li, "Item 2", NULL, NULL, NULL, NULL);
    elm_list_item_append(li, "Item 3 (Which is very long just for testing purposes)", NULL, NULL, NULL, NULL);
    elm_list_go(li);
-   elm_flip_content_back_set(fl, li);
+   elm_object_part_content_set(fl, "back", li);
    evas_object_show(li);
 #endif
 
