@@ -1,7 +1,7 @@
-#include <Elementary.h>
 #ifdef HAVE_CONFIG_H
 # include "elementary_config.h"
 #endif
+#include <Elementary.h>
 #ifndef ELM_LIB_QUICKLAUNCH
 
 typedef struct _Custom_Effect Custom_Effect;
@@ -39,7 +39,6 @@ _custom_op(Elm_Transit_Effect *effect, Elm_Transit *transit, double progress)
 
    EINA_LIST_FOREACH(objs, elist, obj)
       evas_object_resize(obj, w, h);
-
 }
 
 static Elm_Transit_Effect *
@@ -79,7 +78,7 @@ _transit_rotation_translation_color(void *data __UNUSED__, Evas_Object *obj, voi
    elm_transit_effect_translation_add(trans, -70.0, -150.0, 70.0, 150.0);
 
    /* Color Effect */
-   elm_transit_effect_color_add(trans, 100, 255, 100, 255, 200, 50, 200, 50);
+   elm_transit_effect_color_add(trans, 100, 255, 100, 255, 50, 30, 50, 50);
 
    /* Rotation Effect */
    elm_transit_effect_rotation_add(trans, 0.0, 135.0);
@@ -237,18 +236,12 @@ _transit_resizable_flip(void *data, Evas_Object *obj, void *event_info __UNUSED_
 void
 test_transit(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
-   Evas_Object *win, *bg, *bx, *bt, *ic;
+   Evas_Object *win, *bx, *bt, *ic;
    char buf[PATH_MAX];
 
-   win = elm_win_add(NULL, "transit", ELM_WIN_BASIC);
-   elm_win_title_set(win, "Transit");
+   win = elm_win_util_standard_add("transit", "Transit");
    elm_win_autodel_set(win, EINA_TRUE);
 
-   bg = elm_bg_add(win);
-   elm_win_resize_object_add(win, bg);
-   evas_object_size_hint_weight_set(bg, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
-   evas_object_show(bg);
-
    bx = elm_box_add(win);
    evas_object_size_hint_weight_set(bx, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    elm_win_resize_object_add(win, bx);
@@ -288,17 +281,11 @@ test_transit(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_inf
 void
 test_transit2(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
-   Evas_Object *win, *bg, *bt;
+   Evas_Object *win, *bt;
 
-   win = elm_win_add(NULL, "transit2", ELM_WIN_BASIC);
-   elm_win_title_set(win, "Transit 2");
+   win = elm_win_util_standard_add("transit2", "Transit 2");
    elm_win_autodel_set(win, EINA_TRUE);
 
-   bg = elm_bg_add(win);
-   elm_win_resize_object_add(win, bg);
-   evas_object_size_hint_weight_set(bg, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
-   evas_object_show(bg);
-
    bt = elm_button_add(win);
    elm_object_text_set(bt, "Resizing Effect");
    evas_object_show(bt);
@@ -314,17 +301,11 @@ test_transit2(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_in
 void
 test_transit3(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
-   Evas_Object *win, *bg, *bt, *bt2;
+   Evas_Object *win, *bt, *bt2;
 
-   win = elm_win_add(NULL, "transit3", ELM_WIN_BASIC);
-   elm_win_title_set(win, "Transit 3");
+   win = elm_win_util_standard_add("transit3", "Transit 3");
    elm_win_autodel_set(win, EINA_TRUE);
 
-   bg = elm_bg_add(win);
-   elm_win_resize_object_add(win, bg);
-   evas_object_size_hint_weight_set(bg, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
-   evas_object_show(bg);
-
    bt = elm_button_add(win);
    elm_object_text_set(bt, "Front Button - Flip Effect");
    evas_object_show(bt);
@@ -347,17 +328,11 @@ test_transit3(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_in
 void
 test_transit4(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
-   Evas_Object *win, *bg, *bt;
+   Evas_Object *win, *bt;
 
-   win = elm_win_add(NULL, "transit4", ELM_WIN_BASIC);
-   elm_win_title_set(win, "Transit 4");
+   win = elm_win_util_standard_add("transit4", "Transit 4");
    elm_win_autodel_set(win, EINA_TRUE);
 
-   bg = elm_bg_add(win);
-   elm_win_resize_object_add(win, bg);
-   evas_object_size_hint_weight_set(bg, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
-   evas_object_show(bg);
-
    bt = elm_button_add(win);
    elm_object_text_set(bt, "Zoom Effect");
    evas_object_resize(bt, 100, 50);
@@ -374,18 +349,12 @@ test_transit4(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_in
 void
 test_transit5(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
-   Evas_Object *win, *bg, *bt, *bt2, *ic;
+   Evas_Object *win, *bt, *bt2, *ic;
    char buf[PATH_MAX];
 
-   win = elm_win_add(NULL, "transit5", ELM_WIN_BASIC);
-   elm_win_title_set(win, "Transit 5");
+   win = elm_win_util_standard_add("transit5", "Transit 5");
    elm_win_autodel_set(win, EINA_TRUE);
 
-   bg = elm_bg_add(win);
-   elm_win_resize_object_add(win, bg);
-   evas_object_size_hint_weight_set(bg, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
-   evas_object_show(bg);
-
    ic = elm_icon_add(win);
    snprintf(buf, sizeof(buf), "%s/images/rock_01.jpg", elm_app_data_dir_get());
    elm_icon_file_set(ic, buf, NULL);
@@ -420,18 +389,12 @@ test_transit5(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_in
 void
 test_transit6(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
-   Evas_Object *win, *bg, *bt, *bt2, *ic;
+   Evas_Object *win, *bt, *bt2, *ic;
    char buf[PATH_MAX];
 
-   win = elm_win_add(NULL, "transit6", ELM_WIN_BASIC);
-   elm_win_title_set(win, "Transit 6");
+   win = elm_win_util_standard_add("transit6","Transit 6");
    elm_win_autodel_set(win, EINA_TRUE);
 
-   bg = elm_bg_add(win);
-   elm_win_resize_object_add(win, bg);
-   evas_object_size_hint_weight_set(bg, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
-   evas_object_show(bg);
-
    ic = elm_icon_add(win);
    snprintf(buf, sizeof(buf), "%s/images/rock_01.jpg", elm_app_data_dir_get());
    elm_icon_file_set(ic, buf, NULL);
@@ -466,17 +429,11 @@ test_transit6(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_in
 void
 test_transit7(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
-   Evas_Object *win, *bg, *bt, *bt2;
+   Evas_Object *win, *bt, *bt2;
 
-   win = elm_win_add(NULL, "transit7", ELM_WIN_BASIC);
-   elm_win_title_set(win, "Transit 7");
+   win = elm_win_util_standard_add("transit7", "Transit 7");
    elm_win_autodel_set(win, EINA_TRUE);
 
-   bg = elm_bg_add(win);
-   elm_win_resize_object_add(win, bg);
-   evas_object_size_hint_weight_set(bg, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
-   evas_object_show(bg);
-
    bt = elm_button_add(win);
    elm_object_text_set(bt, "Front Button - Resizable Flip Effect");
    evas_object_show(bt);
@@ -499,19 +456,13 @@ test_transit7(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_in
 void
 test_transit8(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
-   Evas_Object *win, *bg, *bt;
+   Evas_Object *win, *bt;
    Elm_Transit *trans;
    Elm_Transit_Effect *effect_context;
 
-   win = elm_win_add(NULL, "transit8", ELM_WIN_BASIC);
-   elm_win_title_set(win, "Transit 8");
+   win = elm_win_util_standard_add("transit8", "Transit 8");
    elm_win_autodel_set(win, EINA_TRUE);
 
-   bg = elm_bg_add(win);
-   elm_win_resize_object_add(win, bg);
-   evas_object_size_hint_weight_set(bg, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
-   evas_object_show(bg);
-
    bt = elm_button_add(win);
    elm_object_text_set(bt, "Button - Custom Effect");
    evas_object_show(bt);
@@ -539,18 +490,12 @@ test_transit8(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_in
 void
 test_transit9(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
-   Evas_Object *win, *bg, *bt, *bt2, *bt3, *bt4;
+   Evas_Object *win, *bt, *bt2, *bt3, *bt4;
    Elm_Transit *trans, *trans2, *trans3, *trans4;
 
-   win = elm_win_add(NULL, "transit9", ELM_WIN_BASIC);
-   elm_win_title_set(win, "Transit 9");
+   win = elm_win_util_standard_add("transit9", "Transit 9");
    elm_win_autodel_set(win, EINA_TRUE);
 
-   bg = elm_bg_add(win);
-   elm_win_resize_object_add(win, bg);
-   evas_object_size_hint_weight_set(bg, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
-   evas_object_show(bg);
-
    bt = elm_button_add(win);
    elm_object_text_set(bt, "Chain 1");
    evas_object_resize(bt, 100, 100);
@@ -610,6 +555,4 @@ test_transit9(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_in
    evas_object_resize(win, 400, 400);
    evas_object_show(win);
 }
-
-
 #endif
