@@ -5,17 +5,11 @@
  * See stdout/stderr for output. Compile with:
  *
  * @verbatim
- * gcc -g `pkg-config --cflags --libs elementary` gengrid_example.c -o gengrid_example
+ * gcc -g gengrid_example.c -o gengrid_example `pkg-config --cflags --libs elementary`
  * @endverbatim
  */
 
 #include <Elementary.h>
-#ifdef HAVE_CONFIG_H
-# include "elementary_config.h"
-#else
-# define __UNUSED__
-# define PACKAGE_DATA_DIR "../../data"
-#endif
 
 typedef struct _Example_Item
 {
@@ -35,13 +29,13 @@ static const char *imgs[9] =
    "wood_01.jpg",
 };
 
-static Elm_Gengrid_Item_Class gic;
-Evas_Object *before_bt, *after_bt;
+static Elm_Gengrid_Item_Class *gic = NULL;
+static Evas_Object *before_bt, *after_bt;
 
 static void
-_on_done(void        *data __UNUSED__,
-         Evas_Object *obj __UNUSED__,
-         void        *event_info __UNUSED__)
+_on_done(void        *data,
+         Evas_Object *obj,
+         void        *event_info)
 {
    elm_exit();
 }
@@ -50,7 +44,7 @@ _on_done(void        *data __UNUSED__,
 static void
 _horizontal_grid(void        *data,
                  Evas_Object *obj,
-                 void        *event_info __UNUSED__)
+                 void        *event_info)
 {
    Evas_Object *grid = data;
 
@@ -61,12 +55,15 @@ _horizontal_grid(void        *data,
 static void
 _always_select_change(void        *data,
                       Evas_Object *obj,
-                      void        *event_info __UNUSED__)
+                      void        *event_info)
 {
    Evas_Object *grid = data;
    Eina_Bool always = elm_check_state_get(obj);
 
-   elm_gengrid_always_select_mode_set(grid, always);
+   if (always)
+     elm_gengrid_select_mode_set(grid, ELM_OBJECT_SELECT_MODE_ALWAYS);
+   else
+     elm_gengrid_select_mode_set(grid, ELM_OBJECT_SELECT_MODE_DEFAULT);
 
    fprintf(stdout, "\"Always select\" mode for gengrid items is now %s\n",
            always ? "on" : "off");
@@ -76,7 +73,7 @@ _always_select_change(void        *data,
 static void
 _bouncing_change(void        *data,
                  Evas_Object *obj,
-                 void        *event_info __UNUSED__)
+                 void        *event_info)
 {
    Evas_Object *grid = data;
    Eina_Bool bounce = elm_check_state_get(obj);
@@ -91,7 +88,7 @@ _bouncing_change(void        *data,
 static void
 _multi_change(void        *data,
               Evas_Object *obj,
-              void        *event_info __UNUSED__)
+              void        *event_info)
 {
    Evas_Object *grid = data;
    Eina_Bool multi = elm_check_state_get(obj);
@@ -106,10 +103,10 @@ _multi_change(void        *data,
 
    if (!multi)
      {
-        Elm_Gengrid_Item *it;
+        Elm_Object_Item *gg_it;
         const Eina_List *selected = elm_gengrid_selected_items_get(grid), *l;
-        EINA_LIST_FOREACH(selected, l, it)
-          elm_gengrid_item_selected_set(it, EINA_FALSE);
+        EINA_LIST_FOREACH(selected, l, gg_it)
+          elm_gengrid_item_selected_set(gg_it, EINA_FALSE);
      }
 }
 
@@ -117,12 +114,15 @@ _multi_change(void        *data,
 static void
 _no_sel_change(void        *data,
                Evas_Object *obj,
-               void        *event_info __UNUSED__)
+               void        *event_info)
 {
    Evas_Object *grid = data;
    Eina_Bool no_sel = elm_check_state_get(obj);
 
-   elm_gengrid_no_select_mode_set(grid, no_sel);
+   if (no_sel)
+     elm_gengrid_select_mode_set(grid, ELM_OBJECT_SELECT_MODE_NONE);
+   else
+     elm_gengrid_select_mode_set(grid, ELM_OBJECT_SELECT_MODE_DEFAULT);
 
    fprintf(stdout, "Selection for gengrid items is now %s\n",
            no_sel ? "disabled" : "enabled");
@@ -131,11 +131,11 @@ _no_sel_change(void        *data,
 /* item selection callback */
 static void
 _grid_sel(void        *data,
-          Evas_Object *obj __UNUSED__,
+          Evas_Object *obj,
           void        *event_info)
 {
    unsigned int x, y;
-   Example_Item *it = elm_gengrid_item_data_get(event_info);
+   Example_Item *it = elm_object_item_data_get(event_info);
 
    elm_gengrid_item_pos_get(event_info, &x, &y);
 
@@ -158,69 +158,69 @@ _item_new(void)
 /* "insert before" callback */
 static void
 _before_bt_clicked(void        *data,
-                   Evas_Object *obj __UNUSED__,
-                   void        *event_info __UNUSED__)
+                   Evas_Object *obj,
+                   void        *event_info)
 {
    Example_Item *it;
    Evas_Object *grid = data;
-   Elm_Gengrid_Item *sel;
+   Elm_Object_Item *sel;
 
    sel = elm_gengrid_selected_item_get(grid);
    if (!sel)
      return;
 
    it = _item_new();
-   elm_gengrid_item_insert_before(grid, &gic, it, sel, _grid_sel, NULL);
+   elm_gengrid_item_insert_before(grid, gic, it, sel, _grid_sel, NULL);
 }
 
 /* "insert after" callback */
 static void
 _after_bt_clicked(void        *data,
-                  Evas_Object *obj __UNUSED__,
-                  void        *event_info __UNUSED__)
+                  Evas_Object *obj,
+                  void        *event_info)
 {
    Example_Item *it;
    Evas_Object *grid = data;
-   Elm_Gengrid_Item *sel;
+   Elm_Object_Item *sel;
 
    sel = elm_gengrid_selected_item_get(grid);
    if (!sel)
      return;
 
    it = _item_new();
-   elm_gengrid_item_insert_after(grid, &gic, it, sel, _grid_sel, NULL);
+   elm_gengrid_item_insert_after(grid, gic, it, sel, _grid_sel, NULL);
 }
 
 /* prepend an item */
 static void
 _prepend_bt_clicked(void        *data,
-                    Evas_Object *obj __UNUSED__,
-                    void        *event_info __UNUSED__)
+                    Evas_Object *obj,
+                    void        *event_info)
 {
    Example_Item *it;
    Evas_Object *grid = data;
 
    it = _item_new();
-   elm_gengrid_item_prepend(grid, &gic, it, _grid_sel, NULL);
+   elm_gengrid_item_prepend(grid, gic, it, _grid_sel, NULL);
 }
 
 /* append an item */
 static void
 _append_bt_clicked(void        *data,
-                   Evas_Object *obj __UNUSED__,
-                   void        *event_info __UNUSED__)
+                   Evas_Object *obj,
+                   void        *event_info)
 {
    Evas_Object *grid = data;
    Example_Item *it = _item_new();
 
-   elm_gengrid_item_append(grid, &gic, it, _grid_sel, NULL);
+   elm_gengrid_item_append(grid, gic, it, _grid_sel, NULL);
 }
 
 /* delete items */
 static void
 _clear_cb(void        *data,
-          Evas_Object *obj __UNUSED__,
-          void        *event_info __UNUSED__)
+          Evas_Object *obj,
+          void        *event_info)
 {
    elm_gengrid_clear(data);
 
@@ -230,48 +230,48 @@ _clear_cb(void        *data,
 /* bring in 1st item */
 static void
 _bring_1st_clicked(void        *data,
-                   Evas_Object *obj __UNUSED__,
-                   void        *event_info __UNUSED__)
+                   Evas_Object *obj,
+                   void        *event_info)
 {
-   Elm_Gengrid_Item *it = elm_gengrid_first_item_get(data);
+   Elm_Object_Item *gg_it = elm_gengrid_first_item_get(data);
 
-   if (!it) return;
+   if (!gg_it) return;
 
-   elm_gengrid_item_bring_in(it);
+   elm_gengrid_item_bring_in(gg_it, ELM_GENGRID_ITEM_SCROLLTO_IN);
 }
 
 /* show last item */
 static void
 _show_last_clicked(void        *data,
-                   Evas_Object *obj __UNUSED__,
-                   void        *event_info __UNUSED__)
+                   Evas_Object *obj,
+                   void        *event_info)
 {
-   Elm_Gengrid_Item *it = elm_gengrid_last_item_get(data);
+   Elm_Object_Item *gg_it = elm_gengrid_last_item_get(data);
 
-   if (!it) return;
+   if (!gg_it) return;
 
-   elm_gengrid_item_show(it);
+   elm_gengrid_item_show(gg_it, ELM_GENGRID_ITEM_SCROLLTO_IN);
 }
 
 /* disable selected item */
 static void
 _toggle_disabled_cb(void        *data,
-                    Evas_Object *obj __UNUSED__,
-                    void        *event_info __UNUSED__)
+                    Evas_Object *obj,
+                    void        *event_info)
 {
-   Elm_Gengrid_Item *it = elm_gengrid_selected_item_get(data);
+   Elm_Object_Item *gg_it = elm_gengrid_selected_item_get(data);
 
-   if (!it) return;
+   if (!gg_it) return;
 
-   elm_gengrid_item_selected_set(it, EINA_FALSE);
-   elm_gengrid_item_disabled_set(it, EINA_TRUE);
+   elm_gengrid_item_selected_set(gg_it, EINA_FALSE);
+   elm_object_item_disabled_set(gg_it, EINA_TRUE);
 }
 
 /* change items' size */
 static void
 _size_changed(void        *data,
               Evas_Object *obj,
-              void        *event_info __UNUSED__)
+              void        *event_info)
 {
    Evas_Object *grid = data;
    int size = elm_spinner_value_get(obj);
@@ -281,8 +281,8 @@ _size_changed(void        *data,
 
 /* item double click callback */
 static void
-_double_click(void        *data __UNUSED__,
-              Evas_Object *obj __UNUSED__,
+_double_click(void        *data,
+              Evas_Object *obj,
               void        *event_info)
 {
    fprintf(stdout, "Double click on item with handle %p\n", event_info);
@@ -290,8 +290,8 @@ _double_click(void        *data __UNUSED__,
 
 /* item long press callback */
 static void
-_long_pressed(void        *data __UNUSED__,
-              Evas_Object *obj __UNUSED__,
+_long_pressed(void        *data,
+              Evas_Object *obj,
               void        *event_info)
 {
    fprintf(stdout, "Long press on item with handle %p\n", event_info);
@@ -301,8 +301,8 @@ _long_pressed(void        *data __UNUSED__,
 /* label fetching callback */
 static char *
 _grid_label_get(void        *data,
-                Evas_Object *obj __UNUSED__,
-                const char  *part __UNUSED__)
+                Evas_Object *obj,
+                const char  *part)
 {
    const Example_Item *it = data;
    char buf[256];
@@ -324,7 +324,7 @@ _grid_content_get(void        *data,
         Evas_Object *icon = elm_bg_add(obj);
         char buf[PATH_MAX];
 
-        snprintf(buf, sizeof(buf), "%s/images/%s", PACKAGE_DATA_DIR,
+        snprintf(buf, sizeof(buf), "%s/images/%s", elm_app_data_dir_get(),
                  it->path);
 
         elm_bg_file_set(icon, buf, NULL);
@@ -347,9 +347,9 @@ _grid_content_get(void        *data,
 
 /* state fetching callback */
 static Eina_Bool
-_grid_state_get(void        *data __UNUSED__,
-                Evas_Object *obj __UNUSED__,
-                const char  *part __UNUSED__)
+_grid_state_get(void        *data,
+                Evas_Object *obj,
+                const char  *part)
 {
    return EINA_FALSE;
 }
@@ -357,7 +357,7 @@ _grid_state_get(void        *data __UNUSED__,
 /* deletion callback */
 static void
 _grid_del(void        *data,
-          Evas_Object *obj __UNUSED__)
+          Evas_Object *obj)
 {
    Example_Item *it = data;
 
@@ -367,9 +367,9 @@ _grid_del(void        *data,
 
 /* scrolling animation stopped callback */
 static void
-_grid_scroll_stopped_cb(void        *data __UNUSED__,
+_grid_scroll_stopped_cb(void        *data,
                         Evas_Object *obj,
-                        void        *event_info __UNUSED__)
+                        void        *event_info)
 {
   int h_pagenumber = 0, v_pagenumber = 0;
   elm_gengrid_current_page_get(obj, &h_pagenumber, &v_pagenumber);
@@ -381,7 +381,7 @@ _grid_scroll_stopped_cb(void        *data __UNUSED__,
 static void
 _h_align_change_cb(void        *data,
                    Evas_Object *obj,
-                   void        *event_info __UNUSED__)
+                   void        *event_info)
 {
    double v_align;
    double val = elm_slider_value_get(obj);
@@ -395,7 +395,7 @@ _h_align_change_cb(void        *data,
 static void
 _v_align_change_cb(void        *data,
                    Evas_Object *obj,
-                   void        *event_info __UNUSED__)
+                   void        *event_info)
 {
    double h_align;
    double val = elm_slider_value_get(obj);
@@ -410,7 +410,7 @@ _v_align_change_cb(void        *data,
 static void
 _page_change_cb(void        *data,
                 Evas_Object *obj,
-                void        *event_info __UNUSED__)
+                void        *event_info)
 {
    double val = elm_slider_value_get(obj);
 
@@ -419,9 +419,9 @@ _page_change_cb(void        *data,
    fprintf(stdout, "Setting grid page's relative size to %f\n", val);
 }
 
-int
-elm_main(int    argc __UNUSED__,
-         char **argv __UNUSED__)
+EAPI_MAIN int
+elm_main(int    argc,
+         char **argv)
 {
    Evas_Object *win, *bg, *grid, *bx, *hbx_1, *hbx_2, *hbx_3, *bt, *ck, *sl,
    *sp;
@@ -430,6 +430,7 @@ elm_main(int    argc __UNUSED__,
 
    srand(time(NULL));
 
+   elm_app_info_set(elm_main, "elementary", "images");
    win = elm_win_add(NULL, "gengrid", ELM_WIN_BASIC);
    elm_win_title_set(win, "Generic Grid Example");
    evas_object_smart_callback_add(win, "delete,request", _on_done, NULL);
@@ -604,11 +605,15 @@ elm_main(int    argc __UNUSED__,
    _page_change_cb(grid, sl, NULL);
    evas_object_smart_callback_add(sl, "changed", _page_change_cb, grid);
 
-   gic.item_style = "default";
-   gic.func.label_get = _grid_label_get;
-   gic.func.content_get = _grid_content_get;
-   gic.func.state_get = _grid_state_get;
-   gic.func.del = _grid_del;
+   if (!gic)
+     {
+        gic = elm_gengrid_item_class_new();
+        gic->item_style = "default";
+        gic->func.text_get = _grid_label_get;
+        gic->func.content_get = _grid_content_get;
+        gic->func.state_get = _grid_state_get;
+        gic->func.del = _grid_del;
+     } // we only create the first time its needed. we dont unref/free
 
    _append_bt_clicked(grid, NULL, NULL);
    _append_bt_clicked(grid, NULL, NULL);
@@ -618,7 +623,8 @@ elm_main(int    argc __UNUSED__,
    evas_object_show(win);
 
    elm_run();
+   elm_shutdown();
+
    return 0;
 }
-
 ELM_MAIN()
