@@ -66,7 +66,8 @@ _event_hook(Evas_Object *obj, Evas_Object *src __UNUSED__, Evas_Callback_Type ty
    if (ev->event_flags & EVAS_EVENT_FLAG_ON_HOLD) return EINA_FALSE;
    if (elm_widget_disabled_get(obj)) return EINA_FALSE;
    if (!wd->video) return EINA_FALSE;
-   if ((!strcmp(ev->keyname, "Left")) || (!strcmp(ev->keyname, "KP_Left")))
+   if ((!strcmp(ev->keyname, "Left")) ||
+       ((!strcmp(ev->keyname, "KP_Left")) && (!ev->string)))
      {
         double current, last;
 
@@ -82,7 +83,8 @@ _event_hook(Evas_Object *obj, Evas_Object *src __UNUSED__, Evas_Callback_Type ty
         ev->event_flags |= EVAS_EVENT_FLAG_ON_HOLD;
         return EINA_TRUE;
      }
-   if ((!strcmp(ev->keyname, "Right")) || (!strcmp(ev->keyname, "KP_Right")))
+   if ((!strcmp(ev->keyname, "Right")) ||
+       ((!strcmp(ev->keyname, "KP_Right")) && (!ev->string)))
      {
         double current, last;
 
@@ -101,7 +103,7 @@ _event_hook(Evas_Object *obj, Evas_Object *src __UNUSED__, Evas_Callback_Type ty
      }
    if (!strcmp(ev->keyname, "space"))
      {
-        if (elm_video_is_playing(wd->video))
+        if (elm_video_is_playing_get(wd->video))
           elm_video_pause(wd->video);
         else
           elm_video_play(wd->video);
@@ -191,7 +193,7 @@ _update_slider(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSE
    Eina_Bool seekable;
 
    if (!wd) return ;
-   seekable = elm_video_is_seekable(wd->video);
+   seekable = elm_video_is_seekable_get(wd->video);
    length = elm_video_play_length_get(wd->video);
    pos = elm_video_play_position_get(wd->video);
 
@@ -225,8 +227,8 @@ _forward(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __
    pos += length * 0.3;
    elm_video_play_position_set(wd->video, pos);
 
-   evas_object_smart_callback_call(player, SIG_FORWARD_CLICKED, NULL);
    edje_object_signal_emit(wd->layout, "elm,button,forward", "elm");
+   evas_object_smart_callback_call(player, SIG_FORWARD_CLICKED, NULL);
 }
 
 static void
@@ -237,8 +239,8 @@ _info(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNU
 
    if (!wd) return ;
 
-   evas_object_smart_callback_call(player, SIG_INFO_CLICKED, NULL);
    edje_object_signal_emit(wd->layout, "elm,button,info", "elm");
+   evas_object_smart_callback_call(player, SIG_INFO_CLICKED, NULL);
 }
 
 static void
@@ -256,8 +258,8 @@ _next(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNU
    pos += length * 0.1;
    elm_video_play_position_set(wd->video, pos);
 
-   evas_object_smart_callback_call(player, SIG_NEXT_CLICKED, NULL);
    edje_object_signal_emit(wd->layout, "elm,button,next", "elm");
+   evas_object_smart_callback_call(player, SIG_NEXT_CLICKED, NULL);
 }
 
 static void
@@ -271,8 +273,8 @@ _pause(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UN
    edje_object_signal_emit(wd->layout, "elm,player,pause", "elm");
    elm_video_pause(wd->video);
 
-   evas_object_smart_callback_call(player, SIG_PAUSE_CLICKED, NULL);
    edje_object_signal_emit(wd->layout, "elm,button,pause", "elm");
+   evas_object_smart_callback_call(player, SIG_PAUSE_CLICKED, NULL);
 }
 
 static void
@@ -283,8 +285,8 @@ _play(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNU
 
    if (!wd) return ;
    elm_video_play(wd->video);
-   evas_object_smart_callback_call(player, SIG_PLAY_CLICKED, NULL);
    edje_object_signal_emit(wd->layout, "elm,button,play", "elm");
+   evas_object_smart_callback_call(player, SIG_PLAY_CLICKED, NULL);
 }
 
 static void
@@ -313,8 +315,8 @@ _rewind(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __U
 
    if (!wd) return ;
    elm_video_play_position_set(wd->video, 0);
-   evas_object_smart_callback_call(player, SIG_REWIND_CLICKED, NULL);
    edje_object_signal_emit(wd->layout, "elm,button,rewind", "elm");
+   evas_object_smart_callback_call(player, SIG_REWIND_CLICKED, NULL);
 }
 
 static void
@@ -325,8 +327,8 @@ _stop(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNU
 
    if (!wd) return ;
 
-   evas_object_smart_callback_call(player, SIG_STOP_CLICKED, NULL);
    edje_object_signal_emit(wd->layout, "elm,button,stop", "elm");
+   evas_object_smart_callback_call(player, SIG_STOP_CLICKED, NULL);
 }
 
 static void
@@ -382,11 +384,9 @@ _cleanup_callback(Widget_Data *wd)
 }
 
 static void
-_track_video(void *data __UNUSED__, Evas *e __UNUSED__, Evas_Object *obj, void *event_info __UNUSED__)
+_track_video(void *data, Evas *e __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
-   Widget_Data *wd = elm_widget_data_get(obj);
-
-   _cleanup_callback(wd);
+   _cleanup_callback(data);
 }
 
 static void
@@ -429,7 +429,7 @@ _player_button_add(Evas_Object *parent, Evas_Object *obj, Evas_Object *layout, c
    return bt;
 }
 
-static const char *
+static char *
 _double_to_time(double value)
 {
    char buf[256];
@@ -449,23 +449,31 @@ _double_to_time(double value)
      snprintf(buf, sizeof(buf), "%02i.%02i",
               ps, pf);
 
-   return eina_stringshare_add(buf);
+   return (char *)eina_stringshare_add(buf);
+}
+
+static void
+_value_free(char *data)
+{
+   eina_stringshare_del(data);
 }
 #endif
 
+#ifdef HAVE_EMOTION
 static void
 _content_set_hook(Evas_Object *obj, const char *part, Evas_Object *content)
 {
    if (part && strcmp(part, "video")) return;
-#ifdef HAVE_EMOTION
    ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
-
+   if (!wd) return;
    double pos, length;
    Eina_Bool seekable;
 
    if (!_elm_video_check(content)) return;
+   if (wd->video == content) return;
 
+   if (wd->video) evas_object_del(wd->video);
    _cleanup_callback(wd);
 
    wd->video = content;
@@ -489,9 +497,9 @@ _content_set_hook(Evas_Object *obj, const char *part, Evas_Object *content)
    wd->emotion = elm_video_emotion_get(wd->video);
    emotion_object_priority_set(wd->emotion, EINA_TRUE);
    evas_object_event_callback_add(wd->video, EVAS_CALLBACK_DEL,
-                                  _track_video, obj);
+                                  _track_video, wd);
 
-   seekable = elm_video_is_seekable(wd->video);
+   seekable = elm_video_is_seekable_get(wd->video);
    length = elm_video_play_length_get(wd->video);
    pos = elm_video_play_position_get(wd->video);
 
@@ -499,7 +507,7 @@ _content_set_hook(Evas_Object *obj, const char *part, Evas_Object *content)
    elm_slider_min_max_set(wd->slider, 0, length);
    elm_slider_value_set(wd->slider, pos);
 
-   if (elm_video_is_playing(wd->video)) edje_object_signal_emit(wd->layout, "elm,player,play", "elm");
+   if (elm_video_is_playing_get(wd->video)) edje_object_signal_emit(wd->layout, "elm,player,play", "elm");
    else edje_object_signal_emit(wd->layout, "elm,player,pause", "elm");
 
    evas_object_smart_callback_add(wd->emotion, "frame_decode",
@@ -516,12 +524,8 @@ _content_set_hook(Evas_Object *obj, const char *part, Evas_Object *content)
                                   _play_finished, obj);
 
    /* FIXME: track info from video */
-#else
-   (void) obj;
-   (void) content;
-#endif
-
 }
+#endif
 
 EAPI Evas_Object *
 elm_player_add(Evas_Object *parent)
@@ -542,6 +546,7 @@ elm_player_add(Evas_Object *parent)
    elm_widget_can_focus_set(obj, EINA_TRUE);
    elm_widget_event_hook_set(obj, _event_hook);
    elm_widget_content_set_hook_set(obj, _content_set_hook);
+   /* TODO: add content_unset and content_get hook */
 
    wd->layout = edje_object_add(e);
    _elm_theme_object_set(obj, wd->layout, "player", "base", "default");
@@ -561,8 +566,8 @@ elm_player_add(Evas_Object *parent)
 
    wd->slider = elm_slider_add(parent);
    elm_widget_sub_object_add(obj, wd->slider);
-   elm_slider_indicator_format_function_set(wd->slider, _double_to_time, eina_stringshare_del);
-   elm_slider_units_format_function_set(wd->slider, _double_to_time, eina_stringshare_del);
+   elm_slider_indicator_format_function_set(wd->slider, _double_to_time, _value_free);
+   elm_slider_units_format_function_set(wd->slider, _double_to_time, _value_free);
    elm_slider_min_max_set(wd->slider, 0, 0);
    elm_slider_value_set(wd->slider, 0);
    elm_object_disabled_set(wd->slider, EINA_TRUE);
@@ -583,9 +588,3 @@ elm_player_add(Evas_Object *parent)
    return NULL;
 #endif
 }
-
-EAPI void
-elm_player_video_set(Evas_Object *player, Evas_Object *video)
-{
-   _content_set_hook(player, "video", video);
-}
