@@ -1,6 +1,7 @@
 #include <Elementary.h>
 #include "elm_priv.h"
 #include "els_icon.h"
+#include <assert.h>
 
 #ifdef ELM_EFREET
 #define NON_EXISTING (void *)-1
@@ -16,29 +17,30 @@ struct _Widget_Data
    Elm_Icon_Lookup_Order lookup_order;
 
 #ifdef HAVE_ELEMENTARY_ETHUMB
-   struct {
-      int id;
+   struct
+     {
+        struct
+          {
+             const char *path;
+             const char *key;
+          } file, thumb;
 
-      struct {
-         const char *path;
-         const char *key;
-      } file, thumb;
+        Ecore_Event_Handler *eeh;
 
-      Ethumb_Exists *exists;
+        Ethumb_Thumb_Format format;
 
-      Ecore_Event_Handler *eeh;
+        Ethumb_Client_Async *request;
 
-      Ethumb_Thumb_Format format;
-
-      Eina_Bool retry : 1;
-   } thumb;
+        Eina_Bool retry : 1;
+     } thumb;
 #endif
 
 #ifdef ELM_EFREET
-   struct {
+   struct
+     {
         int requested_size;
         Eina_Bool use : 1;
-   } freedesktop;
+     } freedesktop;
 #endif
    int in_eval;
    Eina_Bool scale_up : 1;
@@ -59,8 +61,6 @@ struct _Widget_Data
 #ifdef HAVE_ELEMENTARY_ETHUMB
 static Eina_List *_elm_icon_retry = NULL;
 static int _icon_pending_request = 0;
-
-static void _icon_thumb_exists(Ethumb_Client *client __UNUSED__, Ethumb_Exists *thread, Eina_Bool exists, void *data);
 #endif
 
 static const char *widtype = NULL;
@@ -108,17 +108,10 @@ _icon_size_min_get(Evas_Object *icon)
 static void
 _icon_thumb_stop(Widget_Data *wd, void *ethumbd)
 {
-   if (wd->thumb.id >= 0)
+   if (wd->thumb.request)
      {
-        ethumb_client_generate_cancel(ethumbd, wd->thumb.id, NULL, NULL, NULL);
-        wd->thumb.id = -1;
-        _icon_pending_request--;
-     }
-
-   if (wd->thumb.exists)
-     {
-        ethumb_client_thumb_exists_cancel(wd->thumb.exists, _icon_thumb_exists, wd);
-        wd->thumb.exists = NULL;
+        ethumb_client_thumb_async_cancel(ethumbd, wd->thumb.request);
+        wd->thumb.request = NULL;
         _icon_pending_request--;
      }
 
@@ -225,68 +218,34 @@ _icon_thumb_finish(Widget_Data *wd, Ethumb_Client *ethumbd)
 }
 
 static void
-_icon_thumb_cb(void *data,
-               Ethumb_Client *ethumbd,
-               int id,
-               const char *file __UNUSED__,
-               const char *key __UNUSED__,
-               const char *thumb_path,
-               const char *thumb_key,
-               Eina_Bool success)
+_icon_thumb_done(Ethumb_Client *client, const char *thumb_path, const char *thumb_key, void *data)
 {
    Widget_Data *wd = data;
 
-   EINA_SAFETY_ON_FALSE_RETURN(wd->thumb.id == id);
-   wd->thumb.id = -1;
+   assert(wd->thumb.request);
 
    _icon_pending_request--;
+   wd->thumb.request = NULL;
 
-   if (success)
-     {
-        eina_stringshare_replace(&wd->thumb.thumb.path, thumb_path);
-        eina_stringshare_replace(&wd->thumb.thumb.key, thumb_key);
-        wd->thumb.format = ethumb_client_format_get(ethumbd);
+   eina_stringshare_replace(&wd->thumb.thumb.path, thumb_path);
+   eina_stringshare_replace(&wd->thumb.thumb.key, thumb_key);
+   wd->thumb.format = ethumb_client_format_get(client);
 
-        _icon_thumb_finish(wd, ethumbd);
-     }
-   else
-     {
-        ERR("could not generate thumbnail for %s (key: %s)", file, key);
-        _icon_thumb_cleanup(ethumbd);
-     }
+   _icon_thumb_finish(wd, client);
 }
 
 static void
-_icon_thumb_exists(Ethumb_Client *client __UNUSED__, Ethumb_Exists *thread, Eina_Bool exists, void *data)
+_icon_thumb_error(Ethumb_Client *client, void *data)
 {
    Widget_Data *wd = data;
-   Ethumb_Client *ethumbd;
 
-   if (ethumb_client_thumb_exists_check(thread))
-     return ;
+   assert(wd->thumb.request);
 
-   wd->thumb.exists = NULL;
+   _icon_pending_request--;
+   wd->thumb.request = NULL;
 
-   ethumbd = elm_thumb_ethumb_client_get();
-
-   if (exists)
-     {
-        const char *thumb_path, *thumb_key;
-
-        _icon_pending_request--;
-        ethumb_client_thumb_path_get(ethumbd, &thumb_path, &thumb_key);
-        eina_stringshare_replace(&wd->thumb.thumb.path, thumb_path);
-        eina_stringshare_replace(&wd->thumb.thumb.key, thumb_key);
-        wd->thumb.format = ethumb_client_format_get(ethumbd);
-
-        _icon_thumb_finish(wd, ethumbd);
-     }
-   else if ((wd->thumb.id = ethumb_client_generate(ethumbd, _icon_thumb_cb, wd, NULL)) == -1)
-     {
-        ERR("Generate was unable to start !");
-        /* Failed to generate thumbnail */
-        _icon_pending_request--;
-     }
+   ERR("could not generate thumbnail for %s (key: %s)", wd->thumb.file.path, wd->thumb.file.key);
+   _icon_thumb_cleanup(client);
 }
 
 static void
@@ -303,7 +262,7 @@ _icon_thumb_apply(Widget_Data *wd)
    _icon_pending_request++;
    if (!ethumb_client_file_set(ethumbd, wd->thumb.file.path, wd->thumb.file.key)) return ;
    ethumb_client_size_set(ethumbd, _icon_size_min_get(wd->img), _icon_size_min_get(wd->img));
-   wd->thumb.exists = ethumb_client_thumb_exists(ethumbd, _icon_thumb_exists, wd);
+   wd->thumb.request = ethumb_client_thumb_async_get(ethumbd, _icon_thumb_done, _icon_thumb_error, wd);
 }
 
 static Eina_Bool
@@ -663,7 +622,7 @@ elm_icon_add(Evas_Object *parent)
    evas_object_smart_callbacks_descriptions_set(obj, _signals);
 
 #ifdef HAVE_ELEMENTARY_ETHUMB
-   wd->thumb.id = -1;
+   wd->thumb.request = NULL;
 #endif
 
    wd->smooth = EINA_TRUE;
@@ -697,17 +656,10 @@ elm_icon_file_set(Evas_Object *obj, const char *file, const char *group)
    ELM_CHECK_WIDTYPE(obj, widtype) EINA_FALSE;
    Widget_Data *wd = elm_widget_data_get(obj);
    Eina_Bool ret;
-   const char *tmp_file;
-   const char *tmp_group;
 
    if (!wd) return EINA_FALSE;
    EINA_SAFETY_ON_NULL_RETURN_VAL(file, EINA_FALSE);
 
-   _els_smart_icon_file_get(wd->img, &tmp_file, &tmp_group);
-   if ((tmp_file == file || (file && tmp_file && !strcmp(tmp_file, file)))
-       && (group == tmp_group || (group && tmp_group && !strcmp(group, tmp_group))))
-     return EINA_TRUE;
-
 #ifdef ELM_EFREET
    if (!wd->freedesktop.use)
      {
@@ -751,7 +703,7 @@ elm_icon_thumb_set(Evas_Object *obj, const char *file, const char *group)
    eina_stringshare_replace(&wd->thumb.file.path, file);
    eina_stringshare_replace(&wd->thumb.file.key, group);
 
-   if (elm_thumb_ethumb_client_connected())
+   if (elm_thumb_ethumb_client_connected_get())
      {
         _icon_thumb_apply(wd);
         return ;
@@ -943,25 +895,25 @@ elm_icon_no_scale_get(const Evas_Object *obj)
 }
 
 EAPI void
-elm_icon_scale_set(Evas_Object *obj, Eina_Bool scale_up, Eina_Bool scale_down)
+elm_icon_resizable_set(Evas_Object *obj, Eina_Bool size_up, Eina_Bool size_down)
 {
    ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
 
    if (!wd) return;
-   wd->scale_up = scale_up;
-   wd->scale_down = scale_down;
+   wd->scale_up = size_up;
+   wd->scale_down = size_down;
    if (!wd->in_eval) _sizing_eval(obj);
 }
 
 EAPI void
-elm_icon_scale_get(const Evas_Object *obj, Eina_Bool *scale_up, Eina_Bool *scale_down)
+elm_icon_resizable_get(const Evas_Object *obj, Eina_Bool *size_up, Eina_Bool *size_down)
 {
    ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
    if (!wd) return;
-   if (scale_up) *scale_up = wd->scale_up;
-   if (scale_down) *scale_down = wd->scale_down;
+   if (size_up) *size_up = wd->scale_up;
+   if (size_down) *size_down = wd->scale_down;
 }
 
 EAPI void
@@ -1026,11 +978,29 @@ elm_icon_object_get(Evas_Object *obj)
 }
 
 EAPI void
-elm_icon_preload_set(Evas_Object *obj, Eina_Bool disable)
+elm_icon_preload_disabled_set(Evas_Object *obj, Eina_Bool disabled)
 {
    ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
 
    if (!wd) return;
-   _els_smart_icon_preload_set(wd->img, disable);
+   _els_smart_icon_preload_set(wd->img, !!disabled);
+}
+
+EAPI void
+elm_icon_aspect_fixed_set(Evas_Object *obj, Eina_Bool fixed)
+{
+   ELM_CHECK_WIDTYPE(obj, widtype);
+   Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return;
+   _els_smart_icon_aspect_fixed_set(wd->img, fixed);
+}
+
+EAPI Eina_Bool
+elm_icon_aspect_fixed_get(const Evas_Object *obj)
+{
+   ELM_CHECK_WIDTYPE(obj, widtype) EINA_FALSE;
+   Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return EINA_FALSE;
+   return _els_smart_icon_aspect_fixed_get(wd->img);
 }
