@@ -2,6 +2,7 @@
 #include "elm_priv.h"
 
 typedef struct _Widget_Data Widget_Data;
+typedef struct _Elm_Segment_Item Elm_Segment_Item;
 
 struct _Widget_Data
 {
@@ -78,7 +79,11 @@ _del_hook(Evas_Object *obj)
    wd = elm_widget_data_get(obj);
    if (!wd) return;
 
-   EINA_LIST_FREE(wd->seg_items, it) _item_free(it);
+   EINA_LIST_FREE(wd->seg_items, it)
+     {
+        _item_free(it);
+        elm_widget_item_free(it);
+     }
 
    free(wd);
 }
@@ -179,12 +184,8 @@ _item_free(Elm_Segment_Item *it)
    if (wd->selected_item == it) wd->selected_item = NULL;
    if (wd->seg_items) wd->seg_items = eina_list_remove(wd->seg_items, it);
 
-   elm_widget_item_pre_notify_del(it);
-
    if (it->icon) evas_object_del(it->icon);
    if (it->label) eina_stringshare_del(it->label);
-
-   elm_widget_item_del(it);
 }
 
 static void
@@ -330,7 +331,7 @@ _swallow_item_objects(Elm_Segment_Item *it)
 
    if (it->label)
      {
-        edje_object_part_text_set(VIEW(it), "elm.text", it->label);
+        edje_object_part_text_escaped_set(VIEW(it), "elm.text", it->label);
         edje_object_signal_emit(VIEW(it), "elm,state,text,visible", "elm");
      }
    else
@@ -433,6 +434,81 @@ _item_find(const Evas_Object *obj, int idx)
    return it;
 }
 
+static void
+_item_text_set_hook(Elm_Object_Item *it, const char *part, const char *label)
+{
+   Widget_Data *wd;
+   Elm_Segment_Item *item;
+
+   if (part && strcmp(part, "default")) return;
+
+   item = (Elm_Segment_Item *)it;
+   wd = elm_widget_item_data_get(item);
+   if (!wd) return;
+
+   eina_stringshare_replace(&item->label, label);
+   if (item->label)
+     edje_object_signal_emit(VIEW(item), "elm,state,text,visible", "elm");
+   else
+     edje_object_signal_emit(VIEW(item), "elm,state,text,hidden", "elm");
+   edje_object_message_signal_process(VIEW(item));
+   //label can be NULL also.
+   edje_object_part_text_escaped_set(VIEW(item), "elm.text", item->label);
+}
+
+static const char *
+_item_text_get_hook(const Elm_Object_Item *it, const char *part)
+{
+   if (part && strcmp(part, "default")) return NULL;
+   return ((Elm_Segment_Item *)it)->label;
+}
+
+static void
+_item_content_set_hook(Elm_Object_Item *it,
+                       const char *part,
+                       Evas_Object *content)
+{
+   Elm_Segment_Item *item;
+   if (part && strcmp(part, "icon")) return;
+
+   item = (Elm_Segment_Item *)it;
+   if (content == item->icon) return;
+
+   //Remove the existing icon
+   if (item->icon) evas_object_del(item->icon);
+   item->icon = content;
+   if (item->icon)
+     {
+        elm_widget_sub_object_add(VIEW(item), item->icon);
+        edje_object_part_swallow(VIEW(item), "elm.swallow.icon", item->icon);
+        edje_object_signal_emit(VIEW(item), "elm,state,icon,visible", "elm");
+     }
+   else
+     edje_object_signal_emit(VIEW(item), "elm,state,icon,hidden", "elm");
+}
+
+static Evas_Object *
+_item_content_get_hook(const Elm_Object_Item *it, const char *part)
+{
+   if (part && strcmp(part, "icon")) return NULL;
+   return ((Elm_Segment_Item *)it)->icon;
+}
+
+static Eina_Bool
+_item_del_pre_hook(Elm_Object_Item *it)
+{
+   Widget_Data *wd;
+   Elm_Segment_Item *item = (Elm_Segment_Item *)it;
+
+   wd = elm_widget_item_data_get(item);
+   if (!wd) return EINA_FALSE;
+
+   _item_free(item);
+   _update_list(wd);
+
+   return EINA_TRUE;
+}
+
 static Elm_Segment_Item*
 _item_new(Evas_Object *obj, Evas_Object *icon, const char *label)
 {
@@ -445,14 +521,17 @@ _item_new(Evas_Object *obj, Evas_Object *icon, const char *label)
    it = elm_widget_item_new(obj, Elm_Segment_Item);
    if (!it) return NULL;
    elm_widget_item_data_set(it, wd);
+   elm_widget_item_del_pre_hook_set(it, _item_del_pre_hook);
+   elm_widget_item_text_set_hook_set(it, _item_text_set_hook);
+   elm_widget_item_text_get_hook_set(it, _item_text_get_hook);
+   elm_widget_item_content_set_hook_set(it, _item_content_set_hook);
+   elm_widget_item_content_get_hook_set(it, _item_content_get_hook);
 
    VIEW(it) = edje_object_add(evas_object_evas_get(obj));
    edje_object_scale_set(VIEW(it), elm_widget_scale_get(VIEW(it))
                          *_elm_config->scale);
    evas_object_smart_member_add(VIEW(it), obj);
    elm_widget_sub_object_add(obj, VIEW(it));
-   evas_object_clip_set(VIEW(it), evas_object_clip_get(obj));
-
    _elm_theme_object_set(obj, VIEW(it), "segment_control", "item",
                          elm_object_style_get(obj));
    edje_object_mirrored_set(VIEW(it),
@@ -465,7 +544,7 @@ _item_new(Evas_Object *obj, Evas_Object *icon, const char *label)
    else
      edje_object_signal_emit(VIEW(it), "elm,state,text,hidden", "elm");
    edje_object_message_signal_process(VIEW(it));
-   edje_object_part_text_set(VIEW(it), "elm.text", label);
+   edje_object_part_text_escaped_set(VIEW(it), "elm.text", label);
 
    it->icon = icon;
    if (it->icon) elm_widget_sub_object_add(VIEW(it), it->icon);
@@ -518,7 +597,7 @@ elm_segment_control_add(Evas_Object *parent)
    return obj;
 }
 
-EAPI Elm_Segment_Item *
+EAPI Elm_Object_Item *
 elm_segment_control_item_add(Evas_Object *obj, Evas_Object *icon,
                              const char *label)
 {
@@ -535,10 +614,10 @@ elm_segment_control_item_add(Evas_Object *obj, Evas_Object *icon,
    wd->seg_items = eina_list_append(wd->seg_items, it);
    _update_list(wd);
 
-   return it;
+   return (Elm_Object_Item *)it;
 }
 
-EAPI Elm_Segment_Item *
+EAPI Elm_Object_Item *
 elm_segment_control_item_insert_at(Evas_Object *obj, Evas_Object *icon,
                                    const char *label, int idx)
 {
@@ -560,20 +639,7 @@ elm_segment_control_item_insert_at(Evas_Object *obj, Evas_Object *icon,
      wd->seg_items = eina_list_append(wd->seg_items, it);
 
    _update_list(wd);
-   return it;
-}
-
-EAPI void
-elm_segment_control_item_del(Elm_Segment_Item *it)
-{
-   ELM_WIDGET_ITEM_WIDTYPE_CHECK_OR_RETURN(it);
-   Widget_Data *wd;
-
-   wd = elm_widget_item_data_get(it);
-   if (!wd) return;
-
-   _item_free(it);
-   _update_list(wd);
+   return (Elm_Object_Item *)it;
 }
 
 EAPI void
@@ -588,77 +654,27 @@ elm_segment_control_item_del_at(Evas_Object *obj, int idx)
 
    it = _item_find(obj, idx);
    if (!it) return;
-   _item_free(it);
-   _update_list(wd);
+   elm_object_item_del((Elm_Object_Item *)it);
 }
 
 EAPI const char*
 elm_segment_control_item_label_get(const Evas_Object *obj, int idx)
 {
    ELM_CHECK_WIDTYPE(obj, widtype) NULL;
-   Elm_Segment_Item *it;
-
-   it = _item_find(obj, idx);
+   Elm_Segment_Item *it = _item_find(obj, idx);
    if (it) return it->label;
-
    return NULL;
 }
 
-EAPI void
-elm_segment_control_item_label_set(Elm_Segment_Item* it, const char* label)
-{
-   ELM_WIDGET_ITEM_WIDTYPE_CHECK_OR_RETURN(it);
-   Widget_Data *wd;
-
-   wd = elm_widget_item_data_get(it);
-   if (!wd) return;
-
-   eina_stringshare_replace(&it->label, label);
-   if (it->label)
-     edje_object_signal_emit(VIEW(it), "elm,state,text,visible", "elm");
-   else
-     edje_object_signal_emit(VIEW(it), "elm,state,text,hidden", "elm");
-   edje_object_message_signal_process(VIEW(it));
-   //label can be NULL also.
-   edje_object_part_text_set(VIEW(it), "elm.text", it->label);
-}
-
 EAPI Evas_Object *
 elm_segment_control_item_icon_get(const Evas_Object *obj, int idx)
 {
    ELM_CHECK_WIDTYPE(obj, widtype) NULL;
-   Elm_Segment_Item *it;
-
-   it = _item_find(obj, idx);
+   Elm_Segment_Item *it = _item_find(obj, idx);
    if (it) return it->icon;
-
    return NULL;
 }
 
-EAPI void
-elm_segment_control_item_icon_set(Elm_Segment_Item *it, Evas_Object *icon)
-{
-   ELM_WIDGET_ITEM_WIDTYPE_CHECK_OR_RETURN(it);
-
-   //Remove the existing icon
-   if (it->icon)
-     {
-        edje_object_part_unswallow(VIEW(it), it->icon);
-        evas_object_del(it->icon);
-        it->icon = NULL;
-     }
-
-   it->icon = icon;
-   if (it->icon)
-     {
-        elm_widget_sub_object_add(VIEW(it), it->icon);
-        edje_object_part_swallow(VIEW(it), "elm.swallow.icon", it->icon);
-        edje_object_signal_emit(VIEW(it), "elm,state,icon,visible", "elm");
-     }
-   else
-     edje_object_signal_emit(VIEW(it), "elm,state,icon,hidden", "elm");
-}
-
 EAPI int
 elm_segment_control_item_count_get(const Evas_Object *obj)
 {
@@ -672,63 +688,55 @@ elm_segment_control_item_count_get(const Evas_Object *obj)
 }
 
 EAPI Evas_Object*
-elm_segment_control_item_object_get(const Elm_Segment_Item *it)
+elm_segment_control_item_object_get(const Elm_Object_Item *it)
 {
-   ELM_WIDGET_ITEM_WIDTYPE_CHECK_OR_RETURN(it, NULL);
-
+   ELM_OBJ_ITEM_CHECK_OR_RETURN(it, NULL);
    return VIEW(it);
 }
 
-EAPI Elm_Segment_Item*
+EAPI Elm_Object_Item*
 elm_segment_control_item_selected_get(const Evas_Object *obj)
 {
    ELM_CHECK_WIDTYPE(obj, widtype) NULL;
-   Widget_Data *wd;
-
-   wd = elm_widget_data_get(obj);
+   Widget_Data *wd = elm_widget_data_get(obj);
    if (!wd) return NULL;
-
-   return wd->selected_item;
+   return (Elm_Object_Item *) wd->selected_item;
 }
 
 EAPI void
-elm_segment_control_item_selected_set(Elm_Segment_Item *it, Eina_Bool selected)
+elm_segment_control_item_selected_set(Elm_Object_Item *it, Eina_Bool selected)
 {
-   ELM_WIDGET_ITEM_WIDTYPE_CHECK_OR_RETURN(it);
+   ELM_OBJ_ITEM_CHECK_OR_RETURN(it);
    Widget_Data *wd;
+   Elm_Segment_Item *item = (Elm_Segment_Item *)it;
 
-   wd = elm_widget_item_data_get(it);
+   wd = elm_widget_item_data_get(item);
    if (!wd) return;
 
-   if (it == wd->selected_item)
+   if (item == wd->selected_item)
      {
         //already in selected state.
         if (selected) return;
 
         //unselect case
-        _segment_off(it);
+        _segment_off(item);
      }
    else if (selected)
-     _segment_on(it);
+     _segment_on(item);
 
    return;
 }
 
-EAPI Elm_Segment_Item *
+EAPI Elm_Object_Item *
 elm_segment_control_item_get(const Evas_Object *obj, int idx)
 {
    ELM_CHECK_WIDTYPE(obj, widtype) NULL;
-   Elm_Segment_Item *it;
-
-   it = _item_find(obj, idx);
-
-   return it;
+   return (Elm_Object_Item *) _item_find(obj, idx);
 }
 
 EAPI int
-elm_segment_control_item_index_get(const Elm_Segment_Item *it)
+elm_segment_control_item_index_get(const Elm_Object_Item *it)
 {
-   ELM_WIDGET_ITEM_WIDTYPE_CHECK_OR_RETURN(it, -1);
-
-   return it->seg_index;
+   ELM_OBJ_ITEM_CHECK_OR_RETURN(it, -1);
+   return ((Elm_Segment_Item *)it)->seg_index;
 }
