@@ -12,22 +12,25 @@ struct _Widget_Data
 
    Ecore_Timer *delay;
 
-   const char *label;
+   Eina_Hash  *labels;
    const char *units;
    const char *indicator;
 
-   const char *(*indicator_format_func)(double val);
-   void (*indicator_format_free)(const char *str);
+   char *(*indicator_format_func)(double val);
+   void (*indicator_format_free)(char *str);
 
-   const char *(*units_format_func)(double val);
-   void (*units_format_free)(const char *str);
+   char *(*units_format_func)(double val);
+   void (*units_format_free)(char *str);
 
-   double val, val_min, val_max;
+   double val, val_min, val_max, val2;
    Evas_Coord size;
+   Evas_Coord downx, downy;
 
    Eina_Bool horizontal : 1;
    Eina_Bool inverted : 1;
    Eina_Bool indicator_show : 1;
+   Eina_Bool spacer_down : 1;
+   Eina_Bool frozen : 1;
 };
 
 #define ELM_SLIDER_INVERTED_FACTOR (-1.0)
@@ -50,7 +53,9 @@ static void _drag_down(void *data, Evas_Object *obj,
                     const char *emission, const char *source);
 static Eina_Bool _event_hook(Evas_Object *obj, Evas_Object *src,
                              Evas_Callback_Type type, void *event_info);
-static void _spacer_cb(void *data, Evas * e, Evas_Object * obj, void *event_info);
+static void _spacer_down_cb(void *data, Evas * e, Evas_Object * obj, void *event_info);
+static void _spacer_move_cb(void *data, Evas * e, Evas_Object * obj, void *event_info);
+static void _spacer_up_cb(void *data, Evas * e, Evas_Object * obj, void *event_info);
 
 static const char SIG_CHANGED[] = "changed";
 static const char SIG_DELAY_CHANGED[] = "delay,changed";
@@ -90,8 +95,8 @@ _event_hook(Evas_Object *obj, Evas_Object *src __UNUSED__, Evas_Callback_Type ty
    ev = event_info;
    if (ev->event_flags & EVAS_EVENT_FLAG_ON_HOLD) return EINA_FALSE;
    if (elm_widget_disabled_get(obj)) return EINA_FALSE;
-   if ((!strcmp(ev->keyname, "Left"))
-       || (!strcmp(ev->keyname, "KP_Left")))
+   if ((!strcmp(ev->keyname, "Left")) ||
+       ((!strcmp(ev->keyname, "KP_Left")) && (!ev->string)))
      {
         if (!wd->horizontal) return EINA_FALSE;
         if (!wd->inverted) _drag_down(obj, NULL, NULL, NULL);
@@ -99,8 +104,8 @@ _event_hook(Evas_Object *obj, Evas_Object *src __UNUSED__, Evas_Callback_Type ty
         ev->event_flags |= EVAS_EVENT_FLAG_ON_HOLD;
         return EINA_TRUE;
      }
-   else if ((!strcmp(ev->keyname, "Right"))
-            || (!strcmp(ev->keyname, "KP_Right")))
+   else if ((!strcmp(ev->keyname, "Right")) ||
+            ((!strcmp(ev->keyname, "KP_Right")) && (!ev->string)))
      {
         if (!wd->horizontal) return EINA_FALSE;
         if (!wd->inverted) _drag_up(obj, NULL, NULL, NULL);
@@ -108,7 +113,8 @@ _event_hook(Evas_Object *obj, Evas_Object *src __UNUSED__, Evas_Callback_Type ty
         ev->event_flags |= EVAS_EVENT_FLAG_ON_HOLD;
         return EINA_TRUE;
      }
-   else if ((!strcmp(ev->keyname, "Up")) || (!strcmp(ev->keyname, "KP_Up")))
+   else if ((!strcmp(ev->keyname, "Up")) ||
+            ((!strcmp(ev->keyname, "KP_Up")) && (!ev->string)))
      {
         if (wd->horizontal) return EINA_FALSE;
         if (wd->inverted) _drag_up(obj, NULL, NULL, NULL);
@@ -116,7 +122,8 @@ _event_hook(Evas_Object *obj, Evas_Object *src __UNUSED__, Evas_Callback_Type ty
         ev->event_flags |= EVAS_EVENT_FLAG_ON_HOLD;
         return EINA_TRUE;
      }
-   else if ((!strcmp(ev->keyname, "Down")) || (!strcmp(ev->keyname, "KP_Down")))
+   else if ((!strcmp(ev->keyname, "Down")) ||
+            ((!strcmp(ev->keyname, "KP_Down")) && (!ev->string)))
      {
         if (wd->horizontal) return EINA_FALSE;
         if (wd->inverted) _drag_down(obj, NULL, NULL, NULL);
@@ -132,7 +139,7 @@ _del_hook(Evas_Object *obj)
 {
    Widget_Data *wd = elm_widget_data_get(obj);
    if (!wd) return;
-   if (wd->label) eina_stringshare_del(wd->label);
+   if (wd->labels) eina_hash_free(wd->labels);
    if (wd->indicator) eina_stringshare_del(wd->units);
    if (wd->delay) ecore_timer_del(wd->delay);
    free(wd);
@@ -163,6 +170,16 @@ _mirrored_set(Evas_Object *obj, Eina_Bool rtl)
    edje_object_mirrored_set(wd->slider, rtl);
 }
 
+static Eina_Bool
+_labels_foreach_text_set(const Eina_Hash *hash __UNUSED__, const void *key, void *data, void *fdata)
+{
+  Widget_Data *wd = fdata;
+
+  edje_object_part_text_escaped_set(wd->slider, key, data);
+
+  return 1;
+}
+
 static void
 _theme_hook(Evas_Object *obj)
 {
@@ -183,10 +200,10 @@ _theme_hook(Evas_Object *obj)
      edje_object_signal_emit(wd->slider, "elm,state,end,visible", "elm");
    else
      edje_object_signal_emit(wd->slider, "elm,state,end,hidden", "elm");
-   if (wd->label)
+   if (wd->labels)
      {
-        edje_object_part_text_set(wd->slider, "elm.text", wd->label);
-        edje_object_signal_emit(wd->slider, "elm,state,text,visible", "elm");
+        eina_hash_foreach(wd->labels, _labels_foreach_text_set, wd);
+	edje_object_signal_emit(wd->slider, "elm,state,text,visible", "elm");
      }
 
    if (wd->units)
@@ -333,9 +350,9 @@ _units_set(Evas_Object *obj)
    if (!wd) return;
    if (wd->units_format_func)
      {
-        const char *buf;
+        char *buf;
         buf = wd->units_format_func(wd->val);
-        edje_object_part_text_set(wd->slider, "elm.units", buf);
+        edje_object_part_text_escaped_set(wd->slider, "elm.units", buf);
         if (wd->units_format_free) wd->units_format_free(buf);
      }
    else if (wd->units)
@@ -343,10 +360,10 @@ _units_set(Evas_Object *obj)
         char buf[1024];
 
         snprintf(buf, sizeof(buf), wd->units, wd->val);
-        edje_object_part_text_set(wd->slider, "elm.units", buf);
+        edje_object_part_text_escaped_set(wd->slider, "elm.units", buf);
      }
    else
-     edje_object_part_text_set(wd->slider, "elm.units", NULL);
+     edje_object_part_text_escaped_set(wd->slider, "elm.units", NULL);
 }
 
 static void
@@ -356,19 +373,19 @@ _indicator_set(Evas_Object *obj)
    if (!wd) return;
    if (wd->indicator_format_func)
      {
-        const char *buf;
+        char *buf;
         buf = wd->indicator_format_func(wd->val);
-        edje_object_part_text_set(wd->slider, "elm.dragable.slider:elm.indicator", buf);
+        edje_object_part_text_escaped_set(wd->slider, "elm.dragable.slider:elm.indicator", buf);
         if (wd->indicator_format_free) wd->indicator_format_free(buf);
      }
    else if (wd->indicator)
      {
         char buf[1024];
         snprintf(buf, sizeof(buf), wd->indicator, wd->val);
-        edje_object_part_text_set(wd->slider, "elm.dragable.slider:elm.indicator", buf);
+        edje_object_part_text_escaped_set(wd->slider, "elm.dragable.slider:elm.indicator", buf);
      }
    else
-     edje_object_part_text_set(wd->slider, "elm.dragable.slider:elm.indicator", NULL);
+     edje_object_part_text_escaped_set(wd->slider, "elm.dragable.slider:elm.indicator", NULL);
 }
 
 static void
@@ -436,15 +453,18 @@ _drag_down(void *data, Evas_Object *obj __UNUSED__, const char *emission __UNUSE
 }
 
 static void
-_spacer_cb(void *data, Evas *e, Evas_Object *obj __UNUSED__, void *event_info)
+_spacer_down_cb(void *data, Evas *e __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info)
 {
    Widget_Data *wd = elm_widget_data_get(data);
    Evas_Event_Mouse_Down *ev = event_info;
    Evas_Coord x, y, w, h;
-   double button_x, button_y;
+   double button_x = 0.0, button_y = 0.0;
 
+   wd->spacer_down = EINA_TRUE;
+   wd->val2 = wd->val;
    evas_object_geometry_get(wd->spacer, &x, &y, &w, &h);
-   edje_object_part_drag_value_get(wd->slider, "elm.dragable.slider", &button_x, &button_y);
+   wd->downx = ev->canvas.x - x;
+   wd->downy = ev->canvas.y - y;
    if (wd->horizontal)
      {
         button_x = ((double)ev->canvas.x - (double)x) / (double)w;
@@ -458,18 +478,122 @@ _spacer_cb(void *data, Evas *e, Evas_Object *obj __UNUSED__, void *event_info)
         if (button_y < 0) button_y = 0;
      }
    edje_object_part_drag_value_set(wd->slider, "elm.dragable.slider", button_x, button_y);
-   evas_event_feed_mouse_cancel(e, 0, NULL);
-   evas_event_feed_mouse_down(e, 1, EVAS_BUTTON_NONE, 0, NULL);
+   _val_fetch(data);
+   evas_object_smart_callback_call(data, SIG_DRAG_START, NULL);
+   _units_set(data);
+   _indicator_set(data);
+   edje_object_signal_emit(wd->slider, "elm,state,indicator,show", "elm");
 }
 
 static void
-_elm_slider_label_set(Evas_Object *obj, const char *item, const char *label)
+_spacer_move_cb(void *data, Evas *e __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info)
+{
+   Widget_Data *wd = elm_widget_data_get(data);
+   Evas_Event_Mouse_Move *ev = event_info;
+   Evas_Coord x, y, w, h;
+   double button_x = 0.0, button_y = 0.0;
+
+   if  (wd->spacer_down)
+     {
+        Evas_Coord d = 0;
+
+        evas_object_geometry_get(wd->spacer, &x, &y, &w, &h);
+        if (wd->horizontal) d = abs(ev->cur.canvas.x - x - wd->downx);
+        else d = abs(ev->cur.canvas.y - y - wd->downy);
+        if (d > (_elm_config->thumbscroll_threshold - 1))
+          {
+             if (!wd->frozen)
+               {
+                  elm_widget_scroll_freeze_push(data);
+                  wd->frozen = 1;
+               }
+             ev->event_flags &= ~EVAS_EVENT_FLAG_ON_HOLD;
+          }
+
+        if (ev->event_flags & EVAS_EVENT_FLAG_ON_HOLD)
+          {
+             if (wd->spacer_down) wd->spacer_down = EINA_FALSE;
+             _val_fetch(data);
+             evas_object_smart_callback_call(data, SIG_DRAG_STOP, NULL);
+             _units_set(data);
+             _indicator_set(data);
+             if (wd->frozen)
+               {
+                  elm_widget_scroll_freeze_pop(data);
+                  wd->frozen = 0;
+               }
+             edje_object_signal_emit(wd->slider, "elm,state,indicator,hide", "elm");
+             elm_slider_value_set(data, wd->val2);
+             return;
+          }
+        if (wd->horizontal)
+          {
+             button_x = ((double)ev->cur.canvas.x - (double)x) / (double)w;
+             if (button_x > 1) button_x = 1;
+             if (button_x < 0) button_x = 0;
+          }
+        else
+          {
+             button_y = ((double)ev->cur.canvas.y - (double)y) / (double)h;
+             if (button_y > 1) button_y = 1;
+             if (button_y < 0) button_y = 0;
+          }
+        edje_object_part_drag_value_set(wd->slider, "elm.dragable.slider", button_x, button_y);
+        _val_fetch(data);
+        _units_set(data);
+        _indicator_set(data);
+     }
+}
+
+static void
+_spacer_up_cb(void *data, Evas *e __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
+{
+   Widget_Data *wd = elm_widget_data_get(data);
+
+   if (!wd->spacer_down) return;
+   if (wd->spacer_down) wd->spacer_down = EINA_FALSE;
+   _val_fetch(data);
+   evas_object_smart_callback_call(data, SIG_DRAG_STOP, NULL);
+   _units_set(data);
+   _indicator_set(data);
+   if (wd->frozen)
+     {
+        elm_widget_scroll_freeze_pop(data);
+        wd->frozen = 0;
+     }
+   edje_object_signal_emit(wd->slider, "elm,state,indicator,hide", "elm");
+}
+
+static void
+_elm_slider_label_set(Evas_Object *obj, const char *part, const char *label)
 {
    ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
-   if (item && strcmp(item, "default")) return;
+   const char* default_part = "elm.text";
+   const char* real_part;
+
    if (!wd) return;
-   eina_stringshare_replace(&wd->label, label);
+
+   if (!part)
+     real_part = default_part;
+   else
+     real_part = part;
+
+   if (wd->labels)
+     {
+       const char* old_label;
+
+       old_label = eina_hash_find(wd->labels, real_part);
+       if (!old_label)
+	   eina_hash_add(wd->labels, real_part, eina_stringshare_add(label));
+       else
+	 {
+	   eina_stringshare_ref(old_label);
+	   eina_hash_modify(wd->labels, real_part, eina_stringshare_add(label));
+	   eina_stringshare_del(old_label);
+	 }
+     }
+
    if (label)
      {
         edje_object_signal_emit(wd->slider, "elm,state,text,visible", "elm");
@@ -480,18 +604,22 @@ _elm_slider_label_set(Evas_Object *obj, const char *item, const char *label)
         edje_object_signal_emit(wd->slider, "elm,state,text,hidden", "elm");
         edje_object_message_signal_process(wd->slider);
      }
-   edje_object_part_text_set(wd->slider, "elm.text", label);
+
+   edje_object_part_text_escaped_set(wd->slider, real_part, label);
    _sizing_eval(obj);
 }
 
 static const char *
-_elm_slider_label_get(const Evas_Object *obj, const char *item)
+_elm_slider_label_get(const Evas_Object *obj, const char *part)
 {
    ELM_CHECK_WIDTYPE(obj, widtype) NULL;
    Widget_Data *wd = elm_widget_data_get(obj);
-   if (item && strcmp(item, "default")) return NULL;
    if (!wd) return NULL;
-   return wd->label;
+   if (!wd->labels) return NULL;
+
+   if (!part)
+     return eina_hash_find(wd->labels, "elm.text");
+   return eina_hash_find(wd->labels, part);
 }
 
 static void
@@ -611,6 +739,48 @@ _content_unset_hook(Evas_Object *obj, const char *part)
    return NULL;
 }
 
+static void
+_hash_labels_free_cb(void* label)
+{
+  if (label)
+    eina_stringshare_del(label);
+}
+
+static void
+_min_max_set(Evas_Object *obj)
+{
+   char *buf_min = NULL;
+   char *buf_max = NULL;
+
+   Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return;
+   if (wd->units_format_func)
+     {
+        buf_min = wd->units_format_func(wd->val_min);
+        buf_max = wd->units_format_func(wd->val_max);
+     }
+   else if (wd->units)
+     {
+        int length = strlen(wd->units);
+
+        buf_min = alloca(length + 128);
+        buf_max = alloca(length + 128);
+
+        snprintf((char*) buf_min, length + 128, wd->units, wd->val_min);
+        snprintf((char*) buf_max, length + 128, wd->units, wd->val_max);
+     }
+
+   edje_object_part_text_escaped_set(wd->slider, "elm.units.min", buf_min);
+   edje_object_part_text_escaped_set(wd->slider, "elm.units.max", buf_max);
+
+   if (wd->units_format_func && wd->units_format_free)
+     {
+        wd->units_format_free(buf_min);
+        wd->units_format_free(buf_max);
+     }
+}
+
+
 EAPI Evas_Object *
 elm_slider_add(Evas_Object *parent)
 {
@@ -641,6 +811,7 @@ elm_slider_add(Evas_Object *parent)
    wd->val = 0.0;
    wd->val_min = 0.0;
    wd->val_max = 1.0;
+   wd->labels = eina_hash_string_superfast_new(_hash_labels_free_cb);
 
    wd->slider = edje_object_add(e);
    _elm_theme_object_set(obj, wd->slider, "slider", "horizontal", "default");
@@ -658,7 +829,9 @@ elm_slider_add(Evas_Object *parent)
    evas_object_pass_events_set(wd->spacer, EINA_TRUE);
    elm_widget_sub_object_add(obj, wd->spacer);
    edje_object_part_swallow(wd->slider, "elm.swallow.bar", wd->spacer);
-   evas_object_event_callback_add(wd->spacer, EVAS_CALLBACK_MOUSE_DOWN, _spacer_cb, obj);
+   evas_object_event_callback_add(wd->spacer, EVAS_CALLBACK_MOUSE_DOWN, _spacer_down_cb, obj);
+   evas_object_event_callback_add(wd->spacer, EVAS_CALLBACK_MOUSE_MOVE, _spacer_move_cb, obj);
+   evas_object_event_callback_add(wd->spacer, EVAS_CALLBACK_MOUSE_UP, _spacer_up_cb, obj);
    evas_object_smart_callback_add(obj, "sub-object-del", _sub_del, obj);
 
    _mirrored_set(obj, elm_widget_mirrored_get(obj));
@@ -670,36 +843,6 @@ elm_slider_add(Evas_Object *parent)
    return obj;
 }
 
-EAPI void
-elm_slider_label_set(Evas_Object *obj, const char *label)
-{
-   _elm_slider_label_set(obj, NULL, label);
-}
-
-EAPI const char *
-elm_slider_label_get(const Evas_Object *obj)
-{
-   return _elm_slider_label_get(obj, NULL);
-}
-
-EAPI void
-elm_slider_icon_set(Evas_Object *obj, Evas_Object *icon)
-{
-   _content_set_hook(obj, "icon", icon);
-}
-
-EAPI Evas_Object *
-elm_slider_icon_unset(Evas_Object *obj)
-{
-   return _content_unset_hook(obj, "icon");
-}
-
-EAPI Evas_Object *
-elm_slider_icon_get(const Evas_Object *obj)
-{
-   return _content_get_hook(obj, "icon");
-}
-
 EAPI void
 elm_slider_span_size_set(Evas_Object *obj, Evas_Coord size)
 {
@@ -746,6 +889,7 @@ elm_slider_unit_format_set(Evas_Object *obj, const char *units)
         edje_object_signal_emit(wd->slider, "elm,state,units,hidden", "elm");
         edje_object_message_signal_process(wd->slider);
      }
+   _min_max_set(obj);
    _units_set(obj);
    _sizing_eval(obj);
 }
@@ -810,6 +954,7 @@ elm_slider_min_max_set(Evas_Object *obj, double min, double max)
    wd->val_max = max;
    if (wd->val < wd->val_min) wd->val = wd->val_min;
    if (wd->val > wd->val_max) wd->val = wd->val_max;
+   _min_max_set(obj);
    _val_set(obj);
    _units_set(obj);
    _indicator_set(obj);
@@ -880,7 +1025,7 @@ elm_slider_inverted_get(const Evas_Object *obj)
 }
 
 EAPI void
-elm_slider_indicator_format_function_set(Evas_Object *obj, const char *(*func)(double val), void (*free_func)(const char *str))
+elm_slider_indicator_format_function_set(Evas_Object *obj, char *(*func)(double val), void (*free_func)(char *str))
 {
    ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
@@ -891,32 +1036,15 @@ elm_slider_indicator_format_function_set(Evas_Object *obj, const char *(*func)(d
 }
 
 EAPI void
-elm_slider_units_format_function_set(Evas_Object *obj, const char *(*func)(double val), void (*free_func)(const char *str))
+elm_slider_units_format_function_set(Evas_Object *obj, char *(*func)(double val), void (*free_func)(char *str))
 {
    ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
    if (!wd) return;
    wd->units_format_func = func;
    wd->units_format_free = free_func;
-   _indicator_set(obj);
-}
-
-EAPI void
-elm_slider_end_set(Evas_Object *obj, Evas_Object *end)
-{
-   _content_set_hook(obj, "end", end);
-}
-
-EAPI Evas_Object *
-elm_slider_end_unset(Evas_Object *obj)
-{
-   return _content_unset_hook(obj, "end");
-}
-
-EAPI Evas_Object *
-elm_slider_end_get(const Evas_Object *obj)
-{
-   return _content_get_hook(obj, "end");
+   _min_max_set(obj);
+   _units_set(obj);
 }
 
 EAPI void
