@@ -42,12 +42,15 @@ struct _Elm_Tooltip
    Ecore_Timer             *show_timer;
    Ecore_Timer             *hide_timer;
    Ecore_Job               *reconfigure_job;
-   struct {
-      Evas_Coord            x, y, bx, by;
-   } pad;
-   struct {
-      double                x, y;
-   } rel_pos;
+   Evas_Coord               mouse_x, mouse_y;
+   struct
+     {
+        Evas_Coord            x, y, bx, by;
+     } pad;
+   struct
+     {
+        double                x, y;
+     } rel_pos;
    double                   hide_timeout; /* from theme */
    Eina_Bool                visible_lock:1;
    Eina_Bool                changed_style:1;
@@ -67,6 +70,7 @@ static void
 _elm_tooltip_content_changed_hints_cb(void *data, Evas *e __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
    _elm_tooltip_reconfigure_job_start(data);
+   TTDBG("HINTS CHANGED\n");
 }
 
 static void
@@ -83,6 +87,7 @@ _elm_tooltip_obj_move_cb(void *data, Evas *e  __UNUSED__, Evas_Object *obj __UNU
 {
    Elm_Tooltip *tt = data;
    _elm_tooltip_reconfigure_job_start(tt);
+   TTDBG("TT MOVED\n");
 }
 
 static void
@@ -90,12 +95,24 @@ _elm_tooltip_obj_resize_cb(void *data, Evas *e  __UNUSED__, Evas_Object *obj __U
 {
    Elm_Tooltip *tt = data;
    _elm_tooltip_reconfigure_job_start(tt);
+   TTDBG("TT RESIZE\n");
 }
 
 static void
-_elm_tooltip_obj_mouse_move_cb(void *data, Evas *e  __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info  __UNUSED__)
+_elm_tooltip_obj_mouse_move_cb(Elm_Tooltip *tt, Evas *e  __UNUSED__, Evas_Object *obj __UNUSED__, Evas_Event_Mouse_Move *ev)
 {
-   Elm_Tooltip *tt = data;
+   if (tt->mouse_x || tt->mouse_y)
+     {
+        if ((abs(ev->cur.output.x - tt->mouse_x) < 3) &&
+            (abs(ev->cur.output.y - tt->mouse_y) < 3))
+          {
+             TTDBG("MOUSE MOVE REJECTED!\n");
+             return;
+          }
+     }
+   tt->mouse_x = ev->cur.output.x;
+   tt->mouse_y = ev->cur.output.y;
+   TTDBG("MOUSE MOVED\n");
    _elm_tooltip_reconfigure_job_start(tt);
 }
 
@@ -105,9 +122,11 @@ _elm_tooltip_show(Elm_Tooltip *tt)
    _elm_tooltip_show_timer_stop(tt);
    _elm_tooltip_hide_anim_stop(tt);
 
+   TTDBG("TT SHOW\n");
    if (tt->tooltip)
      {
         _elm_tooltip_reconfigure_job_start(tt);
+        TTDBG("RECURSIVE JOB\n");
         return;
      }
    if (tt->free_size)
@@ -135,7 +154,7 @@ _elm_tooltip_show(Elm_Tooltip *tt)
    evas_object_event_callback_add
      (tt->eventarea, EVAS_CALLBACK_RESIZE, _elm_tooltip_obj_resize_cb, tt);
    evas_object_event_callback_add
-     (tt->eventarea, EVAS_CALLBACK_MOUSE_MOVE, _elm_tooltip_obj_mouse_move_cb, tt);
+     (tt->eventarea, EVAS_CALLBACK_MOUSE_MOVE, (Evas_Object_Event_Cb)_elm_tooltip_obj_mouse_move_cb, tt);
 
    tt->changed_style = EINA_TRUE;
    _elm_tooltip_reconfigure_job_start(tt);
@@ -146,6 +165,7 @@ _elm_tooltip_content_del(Elm_Tooltip *tt)
 {
    if (!tt->content) return;
 
+   TTDBG("CONTENT DEL\n");
    evas_object_event_callback_del_full
      (tt->content, EVAS_CALLBACK_CHANGED_SIZE_HINTS,
       _elm_tooltip_content_changed_hints_cb, tt);
@@ -162,6 +182,7 @@ static void
 _elm_tooltip_hide(Elm_Tooltip *tt)
 {
    Evas_Object *del;
+   TTDBG("TT HIDE\n");
    _elm_tooltip_show_timer_stop(tt);
    _elm_tooltip_hide_anim_stop(tt);
    _elm_tooltip_reconfigure_job_stop(tt);
@@ -176,7 +197,7 @@ _elm_tooltip_hide(Elm_Tooltip *tt)
    evas_object_event_callback_del_full
      (tt->eventarea, EVAS_CALLBACK_RESIZE, _elm_tooltip_obj_resize_cb, tt);
    evas_object_event_callback_del_full
-     (tt->eventarea, EVAS_CALLBACK_MOUSE_MOVE, _elm_tooltip_obj_mouse_move_cb, tt);
+     (tt->eventarea, EVAS_CALLBACK_MOUSE_MOVE, (Evas_Object_Event_Cb)_elm_tooltip_obj_mouse_move_cb, tt);
 
    del = tt->tt_win ?: tt->tooltip;
 
@@ -223,6 +244,7 @@ _elm_tooltip_hide_anim_start(Elm_Tooltip *tt)
 {
    double extra = 0;
    if (tt->hide_timer) return;
+   TTDBG("HIDE START\n");
    /* hide slightly faster when in window mode to look less stupid */
    if ((tt->hide_timeout > 0) && tt->tt_win) extra = 0.1;
    edje_object_signal_emit(tt->tooltip, "elm,action,hide", "elm");
@@ -282,12 +304,10 @@ _elm_tooltip_reconfigure(Elm_Tooltip *tt)
              if (str && (!strcmp(str, "enabled")))
                {
                   elm_win_alpha_set(tt->tt_win, EINA_TRUE);
-                  elm_win_transparent_set(tt->tt_win, EINA_TRUE);
                }
              else
                {
                   elm_win_alpha_set(tt->tt_win, EINA_FALSE);
-                  elm_win_transparent_set(tt->tt_win, EINA_FALSE);
                }
           }
 
@@ -304,7 +324,7 @@ _elm_tooltip_reconfigure(Elm_Tooltip *tt)
         str = edje_object_data_get(tt->tooltip, "hide_timeout");
         if (str)
           {
-             tt->hide_timeout = atof(str);
+             tt->hide_timeout = _elm_atof(str);
              if (tt->hide_timeout < 0.0) tt->hide_timeout = 0.0;
           }
 
@@ -447,13 +467,15 @@ _elm_tooltip_reconfigure(Elm_Tooltip *tt)
    else if (ty < 0) ty -= tt->pad.by;
    else if (ty > ch) ty += tt->pad.by;
    TTDBG("PAD (BORDER):  tx=%d,ty=%d\n", tx, ty);
-   if ((tx < 0) || (ty < 0))
+   if (((tx < 0) && (tw < cw)) || ((ty < 0) && (th < ch)))
      {
         TTDBG("POSITIONING FAILED! THIS IS A BUG SOMEWHERE!\n");
+        abort();
         return;
      }
    evas_object_move(tt->tt_win ? : tt->tooltip, tx, ty);
    evas_object_resize(tt->tt_win ? : tt->tooltip, tw, th);
+   TTDBG("FINAL: tx=%d,ty=%d,tw=%d,th=%d\n", tx, ty, tw, th);
    evas_object_show(tt->tooltip);
 
    if (inside_eventarea)
@@ -510,8 +532,8 @@ _elm_tooltip_obj_mouse_in_cb(void *data, Evas *e  __UNUSED__, Evas_Object *obj _
 
    if ((tt->show_timer) || (tt->tooltip)) return;
 
-   tt->show_timer = ecore_timer_add
-     (_elm_config->tooltip_delay, _elm_tooltip_timer_show_cb, tt);
+   tt->show_timer = ecore_timer_add(_elm_config->tooltip_delay, _elm_tooltip_timer_show_cb, tt);
+   TTDBG("MOUSE IN\n");
 }
 
 static void
@@ -525,6 +547,7 @@ _elm_tooltip_obj_mouse_out_cb(Elm_Tooltip *tt, Evas *e  __UNUSED__, Evas_Object
         return;
      }
    _elm_tooltip_hide_anim_start(tt);
+   TTDBG("MOUSE OUT\n");
 }
 
 static void _elm_tooltip_obj_free_cb(void *data, Evas *e  __UNUSED__, Evas_Object *obj, void *event_info  __UNUSED__);
@@ -895,48 +918,17 @@ elm_object_tooltip_style_get(const Evas_Object *obj)
    return tt->style ? tt->style : "default";
 }
 
-/**
- * Get the configured tooltip delay
- *
- * This gets the globally configured tooltip delay in seconds
- *
- * @return The tooltip delay
- * @ingroup Tooltips
- */
-EAPI double
-elm_tooltip_delay_get(void)
-{
-   return _elm_config->tooltip_delay;
-}
-
-/**
- * Set the configured tooltip delay
- *
- * This sets the globally configured delay to tooltip
- *
- * @param delay The delay to show the tooltip
- * @return EINA_TRUE if value is valid and setted
- * @ingroup Tooltips
- */
-EAPI Eina_Bool
-elm_tooltip_delay_set(double delay)
-{
-   if (delay < 0.0) return EINA_FALSE;
-   _elm_config->tooltip_delay = delay;
-   return EINA_TRUE;
-}
-
 /**
  * @brief Disable size restrictions on an object's tooltip
  * @param obj The tooltip's anchor object
  * @param disable If EINA_TRUE, size restrictions are disabled
  * @return EINA_FALSE on failure, EINA_TRUE on success
  *
- * This function allows a tooltip to expand beyond its parant window's canvas.
+ * This function allows a tooltip to expand beyond its parent window's canvas.
  * It will instead be limited only by the size of the display.
  */
 EAPI Eina_Bool
-elm_tooltip_size_restrict_disable(Evas_Object *obj, Eina_Bool disable)
+elm_object_tooltip_window_mode_set(Evas_Object *obj, Eina_Bool disable)
 {
    ELM_TOOLTIP_GET_OR_RETURN(tt, obj, EINA_FALSE);
    return tt->free_size = disable;
@@ -948,11 +940,11 @@ elm_tooltip_size_restrict_disable(Evas_Object *obj, Eina_Bool disable)
  * @return If EINA_TRUE, size restrictions are disabled
  *
  * This function returns whether a tooltip is allowed to expand beyond
- * its parant window's canvas.
+ * its parent window's canvas.
  * It will instead be limited only by the size of the display.
  */
 EAPI Eina_Bool
-elm_tooltip_size_restrict_disabled_get(const Evas_Object *obj)
+elm_object_tooltip_window_mode_get(const Evas_Object *obj)
 {
    ELM_TOOLTIP_GET_OR_RETURN(tt, obj, EINA_FALSE);
    return tt->free_size;
